unit mousefcs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TChangeControlAtCursorPos = procedure(Sender : TObject; OldControl, NewControl : TControl) of object;

  TControlAtCursorPos = class(TComponent)
  private
    FForm : TForm;
    FControl : TControl;
    {instance for EditWndProc}
    ObjectInstance : Pointer;
    {the addres of the EditControl wndproc }
    FFormWndProcAdd : Pointer;

    FOnChange : TChangeControlAtCursorPos;

    procedure SetForm(Value : TForm);
  protected
    {our wndproc}
    procedure FormWndProc(var Message: TMessage);
    {set FForm here}
    procedure Loaded; override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    
    property Control : TControl read FControl;
  published
    property OnChange : TChangeControlAtCursorPos read FOnChange write FOnChange;
  end;

procedure Register;

implementation

constructor TControlAtCursorPos.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  {Create object instance for EditWndProc}
  ObjectInstance := MakeObjectInstance(FormWndProc);
  FForm := Nil;
end;

destructor TControlAtCursorPos.Destroy;
begin
  {Free object instance}
  if (ObjectInstance <> Nil) then
    FreeObjectInstance(ObjectInstance);
  inherited Destroy;
end;

procedure TControlAtCursorPos.Loaded;
begin
  inherited;
  if(Owner <> NIl) And (Owner is TForm) then
    SetForm(TForm(Owner));
end;

Var
  FindControlBusyFlag : Boolean;

procedure TControlAtCursorPos.FormWndProc(var Message: TMessage);
Var
  p : TPoint;
  vControl, ChildvControl : TControl;
begin
  with TMessage(Message) do begin
    if Not FindControlBusyFlag then begin
      FindControlBusyFlag := True;
      if(msg =  WM_MOUSEMOVE) Or (msg = WM_NCMOUSEMOVE) then begin
        p.X := LoWord(lParam);
        p.Y := HiWord(lParam);
      end else begin
        GetCursorPos(p);
        p := FForm.ScreenToClient(p)
      end;
      vControl := FForm.ControlAtPos(p, False);
      if(vControl = Nil) then begin
         p := FForm.ClientToScreen(p);
         vControl := FindVCLWindow(p);
         if(vControl <> Nil) And (vControl <> FForm) then begin
           p := vControl.ScreenToClient(p);
           ChildvControl := TWinControl(vControl).ControlAtPos(p, False);
           if(ChildvControl <> Nil) then
              vControl := ChildvControl;
          end;
      end;
      if(vControl = Nil) then
        FControl := FForm;
      if(FControl <> vControl)then begin
         if Assigned(FOnChange) then
           FOnChange(Nil, FControl, vControl);
         FControl := vControl;
      end;
      FindControlBusyFlag := False;
    end;
    {call the correct wndproc}
    Result := CallWindowProc(FFormWndProcAdd, FForm.Handle, Msg, WParam, LParam);
  end;
end;

procedure TControlAtCursorPos.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  {Is the removed component self component ?}
  if(AComponent = self) And (Operation = opRemove) then
    {call SetFormControl procedure}
   SetForm(Nil);
end;

procedure TControlAtCursorPos.SetForm(Value : TForm);
begin
  if(FForm = Value) then exit;
  {Restore the correct wndproc of the old EditControl}
  if(FForm <> Nil) And (FFormWndProcAdd <> Nil) then begin
    SetWindowLong(FForm.Handle, GWL_WNDPROC, LongInt(FFormWndProcAdd));
    FFormWndProcAdd := Nil;
  end;
  FForm := Value;
  {Save the wndproc of the new EditControl and replace it by our own procedure }
  if(FForm <> Nil) And  Not (csDesigning in FForm.ComponentState) then begin
    FFormWndProcAdd := Pointer(GetWindowLong(FForm.Handle, GWL_WNDPROC));
    SetWindowLong(FForm.Handle, GWL_WNDPROC, LongInt(ObjectInstance));
  end;
end;

procedure Register;
begin
  RegisterComponents('RSDTIPS', [TControlAtCursorPos]);
end;

initialization
  FindControlBusyFlag := False;

end.
